@@ -22,14 +22,13 @@ import logging
 import json
 import pprint
 
-from gi.repository import Gtk
-from gi.repository import GLib
+from gi.repository import Gtk, GLib, GObject, Gio
 
 from gtweak.utils import walk_directories, extract_zip_file, make_combo_list_with_default
-from gtweak.gsettings import GSettingsSetting
+from gtweak.gsettings import GSettingsSetting, GSettingsMissingError, GSettingsFakeSetting
 from gtweak.gshellwrapper import GnomeShellFactory
-from gtweak.tweakmodel import Tweak, TweakGroup, TWEAK_GROUP_THEME
-from gtweak.widgets import ZipFileChooserButton, GSettingsComboTweak, GSettingsComboEnumTweak, GSettingsSwitchTweak, build_label_beside_widget, build_horizontal_sizegroup, build_combo_box_text
+from gtweak.tweakmodel import Tweak, TweakGroup, TWEAK_GROUP_THEME, TWEAK_GROUP_SHELL, TWEAK_SORT_LAST
+from gtweak.widgets import ZipFileChooserButton, GSettingsComboTweak, GSettingsComboEnumTweak, GSettingsSwitchTweak, build_label_beside_widget, build_horizontal_sizegroup, build_combo_box_text, UI_BOX_SPACING
 
 class ShowWindowButtons(GSettingsComboTweak):
     def __init__(self, **options):
@@ -47,7 +46,10 @@ class ShellThemeTweak(Tweak):
     THEME_EXT_NAME = "user-theme@gnome-shell-extensions.gcampax.github.com"
     THEME_GSETTINGS_SCHEMA = "org.gnome.shell.extensions.user-theme"
     THEME_GSETTINGS_NAME = "name"
-    THEME_DIR = os.path.join(GLib.get_home_dir(), ".themes")
+    THEME_GSETTINGS_DIR = os.path.join(GLib.get_user_data_dir(), "gnome-shell", "extensions",
+                                       THEME_EXT_NAME, "schemas")
+    LEGACY_THEME_DIR = os.path.join(GLib.get_home_dir(), ".themes")
+    THEME_DIR = os.path.join(GLib.get_user_data_dir(), "themes")
 
     def __init__(self, **options):
         Tweak.__init__(self, "Shell theme", "Install custom or user themes for gnome-shell", **options)
@@ -64,7 +66,11 @@ class ShellThemeTweak(Tweak):
             if ShellThemeTweak.THEME_EXT_NAME in extensions and extensions[ShellThemeTweak.THEME_EXT_NAME]["state"] == 1:
                 #check the correct gsettings key is present
                 try:
-                    self._settings = GSettingsSetting(ShellThemeTweak.THEME_GSETTINGS_SCHEMA)
+                    if os.path.exists(ShellThemeTweak.THEME_GSETTINGS_DIR):
+                        self._settings = GSettingsSetting(ShellThemeTweak.THEME_GSETTINGS_SCHEMA,
+                                                          schema_dir=ShellThemeTweak.THEME_GSETTINGS_DIR)
+                    else:
+                        self._settings = GSettingsSetting(ShellThemeTweak.THEME_GSETTINGS_SCHEMA)
                     name = self._settings.get_string(ShellThemeTweak.THEME_GSETTINGS_NAME)
 
                     ext = extensions[ShellThemeTweak.THEME_EXT_NAME]
@@ -93,6 +99,7 @@ class ShellThemeTweak(Tweak):
             #      and not themes/, so add it manually later
             dirs = [os.path.join(d, "themes") for d in GLib.get_system_data_dirs()]
             dirs += [ShellThemeTweak.THEME_DIR]
+            dirs += [ShellThemeTweak.LEGACY_THEME_DIR]
 
             valid = walk_directories(dirs, lambda d:
                         os.path.exists(os.path.join(d, "gnome-shell")) and \
@@ -117,6 +124,8 @@ class ShellThemeTweak(Tweak):
 
             self.widget = build_label_beside_widget(self.name, chooser, cb)
             self.widget_for_size_group = cb
+
+            self.widget_sort_hint = TWEAK_SORT_LAST
     
     def _on_file_set(self, chooser):
         f = chooser.get_filename()
@@ -184,6 +193,46 @@ class ShellThemeTweak(Tweak):
         val = combo.get_model().get_value(combo.get_active_iter(), 0)
         self._settings.set_string(ShellThemeTweak.THEME_GSETTINGS_NAME, val)
 
+class StaticWorkspaceTweak(Tweak):
+
+    NUM_WORKSPACES_SCHEMA = "org.gnome.desktop.wm.preferences"
+    NUM_WORKSPACES_KEY = "num-workspaces"
+
+    DYNAMIC_SCHEMA = "org.gnome.shell.overrides"
+    DYNAMIC_KEY = "dynamic-workspaces"
+
+    def __init__(self, **options):
+        Tweak.__init__(self, "Dynamic workspaces", "Disable gnome-shell dynamic workspace management, use static workspaces", **options)
+
+        try:
+            nwsettings = GSettingsSetting(self.NUM_WORKSPACES_SCHEMA, **options)
+        except GSettingsMissingError:
+            self.loaded = False
+            nwsettings = GSettingsFakeSetting()
+
+        try:
+            dsettings = GSettingsSetting(self.DYNAMIC_SCHEMA, **options)
+        except GSettingsMissingError:
+            self.loaded = False
+            dsettings = GSettingsFakeSetting()
+
+        adj = Gtk.Adjustment(1, 1, 99, 1)
+        sb = Gtk.SpinButton(adjustment=adj, digits=0)
+        nwsettings.bind(self.NUM_WORKSPACES_KEY, adj, "value", Gio.SettingsBindFlags.DEFAULT)
+
+        sw = Gtk.Switch()
+        dsettings.bind(self.DYNAMIC_KEY, sw, "active", Gio.SettingsBindFlags.DEFAULT)
+        #sw.bind_property ("active", sb, "sensitive", GObject.BindingFlags.SYNC_CREATE)
+        sb.set_sensitive(not dsettings[self.DYNAMIC_KEY])
+        sw.connect('notify::active', lambda _sw,_param,_sb: _sb.set_sensitive(not _sw.get_active()), sb)
+
+        hb = Gtk.HBox(spacing=UI_BOX_SPACING)
+        hb.pack_start(sw, False, False, 0)
+        hb.pack_start(sb, True, True, 0)
+
+        self.widget = build_label_beside_widget(self.name, hb)
+        self.widget_for_size_group = hb
+
 sg = build_horizontal_sizegroup()
 
 TWEAKS = (
@@ -192,11 +241,14 @@ TWEAKS = (
 
 TWEAK_GROUPS = (
         TweakGroup(
-            _("Shell"),
-            GSettingsSwitchTweak("org.gnome.shell.clock", "show-date", schema_filename="org.gnome.shell.gschema.xml"),
-            GSettingsSwitchTweak("org.gnome.shell.clock", "show-seconds", schema_filename="org.gnome.shell.gschema.xml"),
+            TWEAK_GROUP_SHELL,
+            GSettingsSwitchTweak("org.gnome.desktop.interface", "clock-show-date", schema_filename="org.gnome.desktop.interface.gschema.xml"),
+            GSettingsSwitchTweak("org.gnome.desktop.interface", "clock-show-seconds", schema_filename="org.gnome.desktop.interface.gschema.xml"),
             GSettingsSwitchTweak("org.gnome.shell.calendar", "show-weekdate", schema_filename="org.gnome.shell.gschema.xml"),
             ShowWindowButtons(schema_filename="org.gnome.shell.gschema.xml", size_group=sg),
+            GSettingsSwitchTweak("org.gnome.settings-daemon.plugins.power", "lid-close-suspend-with-external-monitor"),
             GSettingsComboEnumTweak("org.gnome.settings-daemon.plugins.power", "lid-close-battery-action", size_group=sg),
-            GSettingsComboEnumTweak("org.gnome.settings-daemon.plugins.power", "lid-close-ac-action", size_group=sg)),
+            GSettingsComboEnumTweak("org.gnome.settings-daemon.plugins.power", "lid-close-ac-action", size_group=sg),
+            GSettingsSwitchTweak("org.gnome.shell.overrides", "workspaces-only-on-primary"),
+            StaticWorkspaceTweak(size_group=sg)),
 )
