@@ -15,12 +15,18 @@
 # You should have received a copy of the GNU General Public License
 # along with gnome-tweak-tool.  If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, Gdk, Gio, Pango
+import logging
+import os.path
+
+from gi.repository import GLib, Gtk, Gdk, Gio, Pango
 
 from gtweak.tweakmodel import Tweak
-from gtweak.gsettings import GSettingsSetting
+from gtweak.gsettings import GSettingsSetting, GSettingsFakeSetting, GSettingsMissingError
+from gtweak.gtksettings import GtkSettingsManager
 from gtweak.gconf import GConfSetting
 
+UI_BOX_SPACING = 4
+
 def build_label_beside_widget(txt, *widget, **kwargs):
     """
     Builds a HBox containing widgets.
@@ -40,7 +46,7 @@ def build_label_beside_widget(txt, *widget, **kwargs):
     else:
         hbox = Gtk.HBox()
 
-    hbox.props.spacing = 4
+    hbox.props.spacing = UI_BOX_SPACING
     lbl = Gtk.Label(txt)
     lbl.props.ellipsize = Pango.EllipsizeMode.END
     lbl.props.xalign = 0.0
@@ -97,11 +103,23 @@ class _GSettingsTweak(Tweak):
     def __init__(self, schema_name, key_name, **options):
         self.schema_name = schema_name
         self.key_name = key_name
-        self.settings = GSettingsSetting(schema_name, **options)
-        Tweak.__init__(self,
-            options.get("summary",self.settings.schema_get_summary(key_name)),
-            options.get("description",self.settings.schema_get_description(key_name)),
-            **options)
+        try:
+            self.settings = GSettingsSetting(schema_name, **options)
+            Tweak.__init__(self,
+                options.get("summary",self.settings.schema_get_summary(key_name)),
+                options.get("description",self.settings.schema_get_description(key_name)),
+                **options)
+        except GSettingsMissingError, e:
+            self.settings = GSettingsFakeSetting()
+            Tweak.__init__(self,"","")
+            self.loaded = False
+            logging.info("Missing gsettings %s" % (e.message))
+        except KeyError:
+            self.settings = GSettingsFakeSetting()
+            Tweak.__init__(self,"","")
+            self.loaded = False
+            logging.info("Missing gsettings %s (key %s)" % (schema_name, key_name))
+
 
 class GSettingsSwitchTweak(_GSettingsTweak):
     def __init__(self, schema_name, key_name, **options):
@@ -109,7 +127,7 @@ class GSettingsSwitchTweak(_GSettingsTweak):
 
         w = Gtk.Switch()
         self.settings.bind(key_name, w, "active", Gio.SettingsBindFlags.DEFAULT)
-        self.widget = build_label_beside_widget(self.settings.schema_get_summary(key_name), w)
+        self.widget = build_label_beside_widget(self.name, w)
         # never change the size of a switch
         self.widget_for_size_group = None
 
@@ -119,7 +137,7 @@ class GSettingsFontButtonTweak(_GSettingsTweak):
 
         w = Gtk.FontButton()
         self.settings.bind(key_name, w, "font-name", Gio.SettingsBindFlags.DEFAULT)
-        self.widget = build_label_beside_widget(self.settings.schema_get_summary(key_name), w)
+        self.widget = build_label_beside_widget(self.name, w)
         self.widget_for_size_group = w
 
 class GSettingsRangeTweak(_GSettingsTweak):
@@ -131,7 +149,22 @@ class GSettingsRangeTweak(_GSettingsTweak):
 
         w = Gtk.HScale.new_with_range(_min, _max, options.get('adjustment_step', 1))
         self.settings.bind(key_name, w.get_adjustment(), "value", Gio.SettingsBindFlags.DEFAULT)
-        self.widget = build_label_beside_widget(self.settings.schema_get_summary(key_name), w)
+        self.widget = build_label_beside_widget(self.name, w)
+        self.widget_for_size_group = w
+
+class GSettingsSpinButtonTweak(_GSettingsTweak):
+    def __init__(self, schema_name, key_name, **options):
+        _GSettingsTweak.__init__(self, schema_name, key_name, **options)
+
+        #returned variant is range:(min, max)
+        _min, _max = self.settings.get_range(key_name)[1]
+
+        adjustment = Gtk.Adjustment(0, _min, _max, options.get('adjustment_step', 1))
+        w = Gtk.SpinButton()
+        w.set_adjustment(adjustment)
+        w.set_digits(options.get('digits', 0))
+        self.settings.bind(key_name, adjustment, "value", Gio.SettingsBindFlags.DEFAULT)
+        self.widget = build_label_beside_widget(self.name, w)
         self.widget_for_size_group = w
 
 class GSettingsComboEnumTweak(_GSettingsTweak):
@@ -146,7 +179,7 @@ class GSettingsComboEnumTweak(_GSettingsTweak):
         w.connect('changed', self._on_combo_changed)
         self.combo = w
 
-        self.widget = build_label_beside_widget(self.settings.schema_get_summary(key_name), w)
+        self.widget = build_label_beside_widget(self.name, w)
         self.widget_for_size_group = w
 
 
@@ -231,7 +264,7 @@ class GConfFontButtonTweak(_GConfTweak):
         w = Gtk.FontButton()
         w.props.font_name = self.gconf.get_value()
         w.connect("notify::font-name", self._on_fontbutton_changed)
-        self.widget = build_label_beside_widget(self.gconf.schema_get_summary(), w)
+        self.widget = build_label_beside_widget(self.name, w)
         self.widget_for_size_group = w
 
     def _on_fontbutton_changed(self, btn, param):
@@ -248,3 +281,25 @@ class ZipFileChooserButton(Gtk.FileChooserButton):
         #self.set_width_chars(15)
         self.set_local_only(True)
 
+class DarkThemeSwitcher(Tweak):
+    def __init__(self, **options):
+        Tweak.__init__(self, _("Enable dark theme for all applications"),
+                       _("Enable the dark theme hint for all the applications in the session"),
+                       **options)
+
+        self._gtksettings = GtkSettingsManager()
+
+        w = Gtk.Switch()
+        w.set_active(self._gtksettings.get_integer("gtk-application-prefer-dark-theme"))
+
+        w.connect("notify::active", self._on_switch_changed)
+        self.widget = build_label_beside_widget(self.name, w)
+
+    def _on_switch_changed(self, switch, param):
+        active = switch.get_active()
+
+        try:
+            self._gtksettings.set_integer("gtk-application-prefer-dark-theme",
+                                          active)
+        except:
+            self.notify_error(_("Error writing setting"))
